package org.sgine.math

import org.sgine.math.store.NumericStore

trait NumericArray extends Function1[Int, Double] {
	protected def store: NumericStore
	def mutability: Mutability
	
	def apply(index: Int) = store(index)
	
	def apply(offset: Int, f: (Double, Double) => Double, args: Double*) = {
		if (offset + args.length > length) throw new IndexOutOfBoundsException(getClass.getSimpleName + " attempted access to index " + (offset + args.length) + " on NumericArray with length of " + length)
		
		val array = mutability match {
			case Mutability.Mutable => this
			case Mutability.Immutable => copy()
		}
		
		for (index <- 0 until args.length) {
			array.store.modify(offset + index, f(store(offset + index), args(index)))
		}
		
		array
	}
	
	def length = store.length
	
	def update(index: Int, value: Double) = {
		if (index >= length) throw new IndexOutOfBoundsException(getClass.getSimpleName + " only has " + length + ". Attempted to access index " + index + ".")
		
		mutability match {
			case Mutability.Mutable => store.modify(index, value)
			case Mutability.Immutable => {
				val na = copy()
				
				na.store.modify(index, value)
				
				na
			}
		}
	}
	
	def copy(): NumericArray
	
	override def toString() = {
		val b = new StringBuilder()
		
		b.append(getClass.getSimpleName)
		b.append('(')
		for (index <- 0 until length) {
			if (index > 0) {
				b.append(", ")
			}
			b.append(this(index))
		}
		b.append(')')
		
		b.toString
	}
}