package org.sgine.math

import org.sgine.math.store.NumericStore
import org.sgine.math.store.StoreType

trait Vector3 extends NumericArray {
	def x: Double
	def y: Double
	def z: Double
	
	def add(x: Double, y: Double, z: Double): Vector3 = this(0, MathFunctions.+, x, y, z).asInstanceOf[Vector3]
	def +(v: Vector3) = add(v.x, v.y, v.z).asInstanceOf[Vector3]
}

object Vector3 {
	def apply(x: Double = 0.0, y: Double = 0.0, z: Double = 0.0, mutability: Mutability = Mutability.Immutable, storeType: StoreType = StoreType.Default) = {
		val v = storeType match {
			case StoreType.Simple => new SimpleVector3(mutability)
			case _ => new GeneralVector3(storeType.createStore(3), mutability)
		}
		
		v.store.modify(0, x)
		v.store.modify(1, y)
		v.store.modify(2, z)
		
		v
	}
}

private class SimpleVector3(val mutability: Mutability) extends Vector3 with NumericStore {
	val storeType = StoreType.SimpleArray
	def store = this
	
	private var _x: Double = _
	private var _y: Double = _
	private var _z: Double = _
	
	def x = _x
	def y = _y
	def z = _z
	
	override val length = 3
	
	override def apply(index: Int) = {
		index match {
			case 0 => x
			case 1 => y
			case 2 => z
			case _ => throw new IndexOutOfBoundsException("Index out of bounds: " + index + " for length: 3")
		}
	}
	
	def copy() = {
		val v = new SimpleVector3(mutability)
		
		v._x = x
		v._y = y
		v._z = z
		
		v
	}
	
	def modify(index: Int, value: Double) = {
		index match {
			case 0 => _x = value
			case 1 => _y = value
			case 2 => _z = value
			case _ => throw new IndexOutOfBoundsException("Index out of bounds: " + index + " for length: 3")
		}
	}
}

private[math] class GeneralVector3(protected val store: NumericStore, val mutability: Mutability) extends Vector3 {
	def x = apply(0)
	def y = apply(1)
	def z = apply(2)
	
	def copy() = {
		val v = new GeneralVector3(store.storeType.createStore(3), mutability)
		v.store.copyFrom(store)
		v
	}
}